package com.compomics.marc;

import com.compomics.util.experiment.annotation.go.GOFactory;
import com.compomics.util.experiment.biology.AminoAcid;
import com.compomics.util.experiment.biology.Protein;
import com.compomics.util.experiment.identification.SequenceFactory;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;

/**
 *
 * @author Marc
 */
public class AminoAcidComposition {


    private static final String separator = "\t";

    private SequenceFactory sequenceFactory = SequenceFactory.getInstance(1000000);

    public static void main(String[] args) {
        AminoAcidComposition m = new AminoAcidComposition();

        try {
            m.loadDb();
            System.out.println("Database imported");
            m.countAas();
            System.out.println("annotation finished");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void loadDb() throws IOException, FileNotFoundException, ClassNotFoundException {
        File fastaFile = new File("D:\\databases\\uniprot-human_reviewed_december_13_concatenated_target_decoy.fasta");
        sequenceFactory.loadFastaFile(fastaFile, null);
    }
    
    public void countAas() throws IOException, InterruptedException, ClassNotFoundException {
        HashMap<Character, Integer> aaCounts = new HashMap<Character, Integer>();
        for (String accession : sequenceFactory.getAccessions()) {
            if (!sequenceFactory.isDecoyAccession(accession)) {
            Protein protein = sequenceFactory.getProtein(accession);
            char[] aas = protein.getSequence().toCharArray();
            for (char aa : aas) {
                Integer count = aaCounts.get(aa);
                if (count == null) {
                    aaCounts.put(aa, 1);
                } else {
                    aaCounts.put(aa, count+1);
                }
            }
            }
        }
        
        
        File outFile = new File("D:\\databases\\aaCounts.txt");
            BufferedWriter bw = new BufferedWriter(new FileWriter(outFile));
            try {
        
                for (char aa : AminoAcid.getAminoAcids()) {
                    Integer occurrence = aaCounts.get(aa);
                    if (occurrence == null) {
                        occurrence = 0;
                    }
                    bw.write(aa + separator + occurrence);
                    bw.newLine();
                }
        
            } finally {
                bw.close();
            }
    }
    
    
}
