package com.compomics.marc;

import com.compomics.util.experiment.identification.SearchParameters;
import com.compomics.util.experiment.identification.SequenceFactory;
import com.compomics.util.experiment.identification.protein_inference.proteintree.ProteinTree;
import com.compomics.util.gui.waiting.waitinghandlers.ProgressDialogX;
import com.compomics.util.preferences.SequenceMatchingPreferences;
import com.compomics.util.preferences.UtilitiesUserPreferences;
import com.compomics.util.waiting.WaitingHandler;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;

/**
 *
 * @author Marc
 */
public class PeptideMapping {

    
    
    /**
     * The sequence factory.
     */
    private SequenceFactory sequenceFactory = SequenceFactory.getInstance(30000);
    
    /**
     * A dialog to display feedback to the user.
     */
    private WaitingHandler waitingHandler = new ProgressDialogX(false);
    /**
     * The protein tree used to map peptides on protein sequences.
     */
    public ProteinTree proteinTree;

    
    public static void main(String[] args) {

        PeptideMapping m = new PeptideMapping();

        try {
            File databaseFile = new File("C:\\Users\\mva037\\Desktop\\test\\bug lucky", "T312_rat_TD_2013_12_11.fasta");
            m.importSequences(databaseFile);
            m.mapSequence("SHFHPMMLEDSCLTNSTLFSSTALTLLIK");
        } catch (Exception e) {
            e.printStackTrace();
        }
    }
    
    public void mapSequence(String sequence) throws IOException, InterruptedException, ClassNotFoundException, SQLException {
        SequenceMatchingPreferences sequenceMatchingPreferences = new SequenceMatchingPreferences();
        sequenceMatchingPreferences.setSequenceMatchingType(SequenceMatchingPreferences.MatchingType.indistiguishableAminoAcids);
        sequenceMatchingPreferences.setMs2MzTolerance(0.5);
        sequenceMatchingPreferences.setLimitX(0.25);
        HashMap<String, HashMap<String, ArrayList<Integer>>> proteinMapping = sequenceFactory.getDefaultProteinTree().getProteinMapping(sequence, sequenceMatchingPreferences);
        int debug = 1;
    }

    /**
     * Imports sequences from a FASTA file.
     *
     * @param fastaFile FASTA file to process
     */
    public void importSequences(File fastaFile) {

        try {
            waitingHandler.setSecondaryProgressCounterIndeterminate(false);
            sequenceFactory.loadFastaFile(fastaFile, waitingHandler);

            if (waitingHandler.isRunCanceled()) {
                return;
            }

            waitingHandler.resetSecondaryProgressCounter();
            waitingHandler.setSecondaryProgressCounterIndeterminate(true);

            UtilitiesUserPreferences userPreferences = UtilitiesUserPreferences.loadUserPreferences();
            int memoryPreference = userPreferences.getMemoryPreference();
            long fileSize = fastaFile.length();
            long nSequences = sequenceFactory.getNTargetSequences();
            if (!sequenceFactory.isDefaultReversed()) {
                nSequences = sequenceFactory.getNSequences();
            }
            long sequencesPerMb = 1048576 * nSequences / fileSize;
            long availableCachSize = 3 * memoryPreference * sequencesPerMb / 4;
            if (availableCachSize > nSequences) {
                availableCachSize = nSequences;
            } else {
                System.out.println("Warning: PeptideShaker cannot load your FASTA file into memory. This will slow down the processing. "
                        + "Note that using large large databases also reduces the search engine efficiency. "
                        + "Try to either (i) use a smaller database, (ii) increase the memory provided to PeptideShaker, or (iii) improve the reading speed by using an SSD disc. "
                        + "(See also http://code.google.com/p/compomics-utilities/wiki/ProteinInference.)");

            }
            int cacheSize = (int) availableCachSize;
            sequenceFactory.setnCache(cacheSize);

            try {
                proteinTree = sequenceFactory.getDefaultProteinTree(waitingHandler);
            } catch (SQLException e) {
                 System.out.println("Database " + sequenceFactory.getCurrentFastaFile().getName() + " could not be accessed, make sure that the file is not used by another program.");
                e.printStackTrace();
                waitingHandler.setRunCanceled();
            }

            if (!waitingHandler.isRunCanceled()) {
               System.out.println("FASTA file import completed.");
                waitingHandler.increasePrimaryProgressCounter();
            } else {
                sequenceFactory.clearFactory();
            }

        } catch (FileNotFoundException e) {
            System.err.println("File " + fastaFile + " was not found. Please select a different FASTA file.");
            e.printStackTrace();
            waitingHandler.setRunCanceled();
        } catch (IOException e) {
            System.err.println("An error occured while indexing " + fastaFile + ".");
            e.printStackTrace();
            waitingHandler.setRunCanceled();
        } catch (SQLException e) {
            System.err.println("An error occured while indexing " + fastaFile + ".");
            e.printStackTrace();
            waitingHandler.setRunCanceled();
        } catch (InterruptedException e) {
            System.err.println("An error occured while loading " + fastaFile + ".");
            e.printStackTrace();
            waitingHandler.setRunCanceled();
        } catch (IllegalArgumentException e) {
            System.err.println("An error occured while loading " + fastaFile + ".");
            e.printStackTrace();
            waitingHandler.setRunCanceled();
        } catch (ClassNotFoundException e) {
            System.err.println("An error occured while loading " + fastaFile + ".");
            e.printStackTrace();
            waitingHandler.setRunCanceled();
        } catch (NullPointerException e) {
            System.err.println("An error occured while loading " + fastaFile + ".");
            e.printStackTrace();
            waitingHandler.setRunCanceled();
        }
    }
}
